@@ -11,7 +11,7 @@ import sqlite3
 import pickle
 import anndata
 import diffxpy.api as de
-from multiprocessing import Pool
+import multiprocessing
 from itertools import repeat
 from swan_vis.utils import *
 from swan_vis.talon_utils import *
@@ -161,7 +161,7 @@ class SwanGraph(Graph):
 			if ftype == 'gtf':
 				self.create_dfs_gtf(fname)
 			elif ftype == 'db':
-				self.create_dfs_db(fname, annot, whitelist, 'hepg2_1')
+				self.create_dfs_db(fname, annot, whitelist, dataset_name)
 
 			# add column to each df to indicate where data came from
 			self.loc_df[col] = True
@@ -175,7 +175,7 @@ class SwanGraph(Graph):
 			if ftype == 'gtf':
 				temp.create_dfs_gtf(fname)
 			elif ftype == 'db':
-				temp.create_dfs_db(fname, annot, whitelist, 'hepg2_1')
+				temp.create_dfs_db(fname, annot, whitelist, dataset_name)
 			self.merge_dfs(temp, col)
 
 		# remove isms if we have access to that information
@@ -986,6 +986,8 @@ class SwanGraph(Graph):
 
 				ir_genes (list of str): A list of gene ids from the SwanGraph with 
 					at least one novel intron retention event
+				ir_transcripts (list of str): A list of transcript ids from the 
+					SwanGraph with at least one novel intron retention event
 		"""
 
 		# get only novel edges
@@ -1004,6 +1006,7 @@ class SwanGraph(Graph):
 		# for each edge, see if the subgraph between the edge vertices 
 		# contains an exonic edge  
 		ir_genes = []
+		ir_transcripts = []
 		for i, eid in enumerate(edge_ids):
 			sub_nodes = [i for i in range(eid[0]+1,eid[1])]
 			sub_G = self.G.subgraph(sub_nodes)
@@ -1035,13 +1038,17 @@ class SwanGraph(Graph):
 						for cand_eid in sub_edges.index:
 							temp_df = cand_g_df[[cand_eid in vertex_to_edge_path(x) \
 									for x in cand_g_df.path.values.tolist()]]
+							tids = temp_df.tid.tolist()
 							if len(temp_df.index) > 0:
 								ir_genes.append(gid)
+								ir_transcripts.extend(tids)
 
 		print('Found {} novel ir events from {} genes.'.format(len(ir_genes), 
 			len(list(set(ir_genes)))))
 		ir_genes = list(set(ir_genes))
-		return ir_genes
+		ir_transcripts = list(set(ir_transcripts))
+
+		return ir_genes, ir_transcripts
 
 	def find_es_genes(self):
 		"""
@@ -1052,6 +1059,8 @@ class SwanGraph(Graph):
 
 				es_genes (list of str): A list of gene ids from the SwanGraph with 
 					at least one novel exon skipping event
+				es_transcripts (list of str): A list of transcript ids from the 
+					SwanGraph with at least one novel exon skipping event
 		"""
 
 		# get only novel edges
@@ -1070,6 +1079,7 @@ class SwanGraph(Graph):
 		# for each edge, see if the subgraph between the edge vertices 
 		# contains an exonic edge
 		es_genes = []
+		es_transcripts = []
 		for eid in edge_ids:
 			sub_nodes = [i for i in range(eid[0]+1,eid[1])]
 			sub_G = self.G.subgraph(sub_nodes)
@@ -1101,13 +1111,17 @@ class SwanGraph(Graph):
 						for skip_eid in sub_edges.index:
 							temp_df = skip_g_df[[skip_eid in vertex_to_edge_path(x) \
 									for x in skip_g_df.path.values.tolist()]]
+							tids = temp_df.tid.tolist()
 							if len(temp_df.index) > 0:
 								es_genes.append(gid)
+								es_transcripts.extend(tids)
 
 		print('Found {} novel es events from {} genes.'.format(len(es_genes),
 			len(list(set(es_genes)))))
 		es_genes = list(set(es_genes))
-		return es_genes
+		es_transcripts = list(set(es_transcripts))
+
+		return es_genes, es_transcripts
 
 	def de_gene_test(self, dataset_groups):
 		""" 
@@ -1667,7 +1681,8 @@ class SwanGraph(Graph):
 				   indicate_dataset=False, 
 				   indicate_novel=False,
 				   browser=False,
-				   order='expression'):
+				   order='expression',
+				   threads=1):
 		"""
 		Generates a PDF report for a given gene or list of genes according
 		to the user's input.
@@ -1741,6 +1756,10 @@ class SwanGraph(Graph):
 						'tes': genomic coordinate of transcription end site
 					Default: 'expression' if abundance information is present,
 							 'tid' if not
+
+				threads (int): Number of threads to use. Multithreading is 
+					recommended when making a large number of gene reports.
+					Default: 1. 
 		"""
 
 		# check to see if input genes are in the graph
@@ -1800,7 +1819,9 @@ class SwanGraph(Graph):
 
 		# make sure de has been run if needed
 		if include_qvals:
-			self.check_de('transcript')
+			if not self.check_de('transcript'):
+				raise Exception('Differential transcript expression test needed '
+					'to use include_qvals. Run de_transcript_test.')
 			de_df = self.det_test.copy(deep=True)
 			t_df = reset_dupe_index(t_df, 'tid')
 			t_df['significant'] = False
@@ -1860,95 +1881,25 @@ class SwanGraph(Graph):
 		else:
 			report_type = 'browser'
 
-		# parallel
-		# launch report jobs on different threads
-		with Pool() as pool:
-			pool.starmap(_create_gene_report, zip(gids, repeat(self), repeat(t_df),
-				repeat(datasets), repeat(data_type), repeat(prefix), repeat(indicate_dataset),
-				repeat(indicate_novel), repeat(browser), repeat(report_type),
-				repeat(novelty), repeat(heatmap), repeat(include_qvals)))
+		# make sure number of threads is compatible with the system
+		max_cores = multiprocessing.cpu_count()
+		if threads > max_cores:
+			threads = max_cores
 
-		# # not parallel
-		# # loop through each gid and create the report
-		# for gid in gids:
-
-		# 	report_tids = t_df.loc[t_df.gid == gid, 'tid'].tolist()
-
-		# 	# plot each transcript with these settings
-		# 	print('Plotting transcripts for {}'.format(gid))
-		# 	self.plot_each_transcript(report_tids, prefix,
-		# 							  indicate_dataset,
-		# 							  indicate_novel,
-		# 							  browser=browser)
-
-		# 	# if we're plotting tracks, we need a scale as well
-		# 	if not browser:
-		# 		report_type = 'swan'
-		# 	else:
-		# 		self.pg.plot_browser_scale()
-		# 		self.save_fig(prefix+'_browser_scale.png')
-		# 		report_type = 'browser'
-
-		# 	# subset on gene
-		# 	gid_t_df = t_df.loc[t_df.gid == gid].copy(deep=True)
-
-		# 	if heatmap:
-		# 		# take log2(tpm) and gene-normalize 
-		# 		count_cols = ['{}_counts'.format(d) for d in datasets]
-		# 		log_cols = ['{}_log_tpm'.format(d) for d in datasets]
-		# 		norm_log_cols = ['{}_norm_log_tpm'.format(d) for d in datasets]
-		# 		gid_t_df[log_cols] = np.log2(gid_t_df[count_cols]+1)
-		# 		max_val = max(gid_t_df[log_cols].max().tolist())
-		# 		min_val = min(gid_t_df[log_cols].min().tolist())
-		# 		gid_t_df[norm_log_cols] = (gid_t_df[log_cols]-min_val)/(max_val-min_val)
-
-		# 		# create a colorbar 
-		# 		plt.rcParams.update({'font.size': 20})
-		# 		fig, ax = plt.subplots(figsize=(14, 1.5))
-		# 		fig.subplots_adjust(bottom=0.5)
-		# 		fig.patch.set_visible(False)
-		# 		ax.patch.set_visible(False)
-
-		# 		cmap = plt.get_cmap('Spectral_r')
-		# 		norm = mpl.colors.Normalize(vmin=min_val, vmax=max_val)
-
-		# 		cb = mpl.colorbar.ColorbarBase(ax,
-		# 										cmap=cmap,
-		# 										norm=norm,
-		# 										orientation='horizontal')
-		# 		cb.set_label('log2(TPM)')
-		# 		plt.savefig(prefix+'_colorbar_scale.png', format='png', dpi=200)
-		# 		plt.clf()
-		# 		plt.close()
-
-		# 	# create report
-		# 	print('Generating report for {}'.format(gid))
-		# 	pdf_name = create_fname(prefix, 
-		# 				 indicate_dataset,
-		# 				 indicate_novel,
-		# 				 browser,
-		# 				 ftype='report',
-		# 				 gid=gid)
-		# 	report = Report(prefix,
-		# 					report_type,
-		# 					datasets,
-		# 					data_type,
-		# 					novelty=novelty,
-		# 					heatmap=heatmap)
-		# 	report.add_page()
-
-		# 	# loop through each transcript and add it to the report
-		# 	for tid in report_tids:
-		# 		entry = gid_t_df.loc[tid]
-		# 		## TODO would be faster if I didn't have to compute these names twice....
-		# 		## ie once in plot_each_transcript and once here
-		# 		fname = create_fname(prefix,
-		# 							 indicate_dataset,
-		# 							 indicate_novel, 
-		# 							 browser,
-		# 							 tid=entry.tid)
-		# 		report.add_transcript(entry, fname)
-		# 	report.write_pdf(pdf_name)
+		# if there are fewer than 10 genes, the overhead for multiprocessing 
+		# takes longer 
+		if len(gids) < 10:
+			for gid in gids:
+				_create_gene_report(gid, self, t_df, datasets, data_type,
+					prefix, indicate_dataset, indicate_novel, browser,
+					report_type, novelty, heatmap, include_qvals) 
+		else:
+			# launch report jobs on different threads
+			with multiprocessing.Pool(threads) as pool:
+				pool.starmap(_create_gene_report, zip(gids, repeat(self), repeat(t_df),
+					repeat(datasets), repeat(data_type), repeat(prefix), repeat(indicate_dataset),
+					repeat(indicate_novel), repeat(browser), repeat(report_type),
+					repeat(novelty), repeat(heatmap), repeat(include_qvals)))
 
 	##########################################################################
 	############################# Error handling #############################
